#include "rpartS.h"
#include "func_table.h"
#include "node.h"
#include "rpart.h"
#include "rpartproto.h"
#include <stdio.h>
#include <math.h>

/* SCCS @(#)anova.c	1.5 12/13/99  */
/*
** The four routines for anova splitting
*/
#define LEFT  (-1)     /*used for the variable "extra" in nodes */
#define RIGHT  1
#define MISSING 0

static double *mean, *sums;
static double *wts;
static int *countn;
static int *tsplit;

int anovainit(int n,        double *y[],  int maxcat, char **error, 
	      double *parm, int *size,    int who,    double *wt)
    {
    if (who==1 && maxcat >0) {
	graycode_init0(maxcat);
	countn  = (int *)ALLOC(2*maxcat, sizeof(int));
	tsplit  = countn + maxcat;
	mean   = (double *)ALLOC(3*maxcat, sizeof(double));
	wts    = mean + maxcat;
	sums   = wts + maxcat;
	}
    *size =1;
    return(0);
    }
/*
** The anova evaluation function.  Return the mean and the ss.
*/
void anovass(int n, double *y[], double *value, double *risk,
	     double *wt) {
    int i;
    double temp, twt;
    double mean, ss;

    temp =0;
    twt  =0;   		/* sum of the weights */
    for (i=0; i<n; i++) {
	temp += *y[i] * wt[i];
	twt  += wt[i];
	}
    mean = temp/twt;

    ss =0;
    for (i=0; i<n; i++) {
	temp = *y[i] - mean;
	ss += temp*temp* wt[i];
	}

    *value = mean;
    *risk = ss;
    }

/*
** The anova splitting function.  Find that split point in x such that
**  the sum of squares of y within the two groups is decreased as much
**  as possible.  It is not necessary to actually calculate the SS, the
**  improvement involves only means in the two groups.
*/
void anova(int n,    double *y[],     FLOAT *x,     int nclass, 
	   int edge, double *improve, FLOAT *split, int *csplit, 
	   double myrisk,             double *wt)
    {
    int i,j;
    double temp;
    double left_sum, right_sum;
    double left_wt, right_wt;
    int    left_n,  right_n;
    double grandmean, best;
    int direction;
    int where;
    int ncat;

    /*
    ** The improvement of a node is SS - (SS_L + SS_R), where
    **   SS = sum of squares in a node = \sum w_i (x_i - \bar x)^2, where
    ** of course \bar x is a weighted mean \sum w_i x_i / \sum w_i
    ** Using the identity 
    **    \sum w_i(x_ - \bar x)^2 = \sum w_i (x_i-c)^2 - (\sum w_i)(c-\bar x)^2
    ** the improvement = w_l*(left mean - grand mean)^2 
    **                  +w_r*(right mean- grand mean)^2
    ** where w_l is the sum of weights in the left node, w_r similarly. 
    */
    right_wt =0;
    right_n  = n;
    right_sum =0;
    for (i=0; i<n; i++) {
	right_sum += *y[i] * wt[i];
	right_wt  += wt[i];
	}
    grandmean = right_sum/right_wt;

    if (nclass==0) {   /* continuous predictor */
        left_sum=0;   /* No data in left branch, to start */
	left_wt =0;   left_n =0;
	right_sum=0;  /*after subracting grand mean, it's zero */
	best    = 0;
	for (i=0; right_n>edge; i++) {
	    left_wt += wt[i];  
	    right_wt -= wt[i];
	    left_n++;
	    right_n--;
	    temp = (*y[i] - grandmean) * wt[i];
	    left_sum  +=temp;
	    right_sum -=temp;
	    if (x[i+1] !=x[i] &&  left_n>=edge) {
		temp = left_sum*left_sum/left_wt  +
			    right_sum*right_sum/right_wt;
		if (temp > best) {
		    best=temp;
		    where =i;
		    if (left_sum < right_sum) direction = LEFT;
				      else    direction = RIGHT;
		    }
		}
	    }

	*improve =  best/ myrisk;
	if (best>0) {   /* found something */
	    csplit[0] = direction;
	    *split = (x[where] + x[where+1]) /2;
	    }
	}

    /* 
    ** Categorical predictor 
    */
    else {
	for (i=0; i<nclass; i++) {
	    sums[i] =0;
	    countn[i]=0;
	    wts[i] =0;
	    }

	/* rank the classes by their mean y value */
	for (i=0; i<n; i++) {
	    j = x[i] -1;
	    countn[j]++;
	    wts[j] += wt[i];
	    sums[j] += (*y[i] - grandmean) * wt[i];
	    }
	for (i=0; i<nclass; i++)  {
	    if (countn[i] >0) {
		tsplit[i] = RIGHT;
		mean[i] = sums[i]/ wts[i];
		}
	    else tsplit[i] = 0;
	    }
	graycode_init2(nclass, countn, mean);

	/*
	** Now find the split that we want
	*/
	left_wt =0; 
	left_sum=0; right_sum=0;  
	left_n = 0; 
	best =0;
	where =0;
	while((j=graycode()) < nclass) {
	    tsplit[j] = LEFT;
	    left_n += countn[j];
	    right_n-= countn[j];
	    left_wt += wts[j];
	    right_wt-= wts[j];
	    left_sum += sums[j];
	    right_sum-= sums[j];
	    if (left_n>=edge  &&  right_n>=edge) {
		temp = left_sum *left_sum /left_wt  + 
		       right_sum*right_sum/right_wt;
		if (temp > best) {
		    best=temp;
		    if ((left_sum/left_wt) > (right_sum/right_wt)) {
			for (i=0; i<nclass; i++) csplit[i] = -tsplit[i];
			}
		    else {
			for (i=0; i<nclass; i++) csplit[i] = tsplit[i];
			}
		    }
		}
	    }

	*improve = best/ myrisk;      /* % improvement */
	}
    }
/* SCCS @(#)anovapred.c	1.3  02/08/98 */
/*
** The error function for anova splitting
*/
static
double anovapred(double *y, double *yhat)
    {
    double temp;

    temp = y[0] - *yhat;
    return(temp*temp);
    }
/* SCCS @(#)branch.c	1.4 12/13/99 */
/*
** Walk an observation 'one more split' down the tree.  If there are no
**   more splits, return 0, otherwise return the address of the new node.
** A return of zero also comes about if surrogates aren't being used, and I
**   hit a missing value.
*/

static
struct node *branch(struct node *tree, int obs)
    {
    int i, j, dir;
    struct node *me;
    struct split *tsplit;
    FLOAT **xdata;
    long   **sorts;
    if (tree->leftson ==0) return(0);

    me = tree;
    xdata = rp.xdata;
    sorts = rp.sorts;
    /*
    ** choose left or right son
    **   this may use lots of surrogates before we're done
    */
    tsplit = me->primary;
    j= tsplit->var_num;
    if (rp.numcat[j]==0) { /* continuous */
	for (i=0; i<rp.n; i++) {
	    if (sorts[j][i] == obs) {  /* found the match */
		if (xdata[j][i] < tsplit->spoint)
			dir =  tsplit->csplit[0];
		   else dir = -tsplit->csplit[0];
		goto down;
		}
	    }
	}
    else {
	dir = (tsplit->csplit)[(int)xdata[j][obs] -1];
	if (dir !=0) goto down;
	}

    if (rp.usesurrogate==0) return(0);
    /*
    ** use the surrogates
    */
    for (tsplit= me->surrogate; tsplit !=0; tsplit= tsplit->nextsplit) {
	j = tsplit->var_num;
	if (rp.numcat[j]==0) {
	    for (i=0; i<rp.n; i++) {
		if (sorts[j][i] == obs) {
		    if (xdata[j][i] < tsplit->spoint)
			    dir =  tsplit->csplit[0];
		       else dir = -tsplit->csplit[0];
		    goto down;
		    }
		}
	    }
	else {
	    dir = (tsplit->csplit)[(int)xdata[j][obs] -1];
	    if (dir !=0) goto down;
	    }
	}


    if (rp.usesurrogate <2) return(0);
    /*
    ** split it by default
    */
    dir = me->lastsurrogate;

down:if (dir==LEFT) return(me->leftson);
     else           return(me->rightson);
    }
/* SCCS @(#)bsplit.c	1.5 12/13/99 */
/*
** The routine which will find the best split for a node
**
** Input :      node
**              node number
**
** Output:      Fills in the node's
**                      primary splits
**                      competitor splits
*/

static
void bsplit(struct node *me, int nodenum)
    {
    int i, j, k;
    int nc;
    double improve;
    FLOAT split;
    struct split *tsplit;
    long *index;
    int  *which;
    FLOAT *xtemp;  /*these 3 because I got tired of typeing "rp.xtemp", etc*/
    double **ytemp;
    double *wtemp;  

    which = rp.which;
    xtemp = rp.xtemp;
    ytemp = rp.ytemp;
    wtemp = rp.wtemp;

    /*
    ** test out the variables 1 at at time
    */
    me->primary =0;
    for (i=0; i<rp.nvar; i++) {
	index = rp.sorts[i];
	nc = rp.numcat[i];
	/* extract x and y data */
	k=0;
	for (j=0; j<rp.n; j++)
	    if (index[j] >=0 && which[index[j]]== nodenum) {
		xtemp[k] = rp.xdata[i][j];
		ytemp[k] = rp.ydata[index[j]];
		wtemp[k] = rp.wt[index[j]];
		k++;
		}
	if (k==0 ||
	  (nc==0 &&  xtemp[0]==xtemp[k-1])) continue;  /*no place to split */

	(*rp_choose)(k, ytemp, xtemp, nc, rp.min_node, &improve,
			     &split, rp.csplit, me->risk, wtemp);
	if (improve >0) {
	    tsplit = insert_split(&(me->primary), nc, improve, rp.maxpri);
	    if (tsplit !=0) {
		tsplit->improve = improve;
		tsplit->var_num = i;
		tsplit->spoint  = split;
		tsplit->count   = k;
		if (nc ==0) {
		    tsplit->spoint = split;
		    tsplit->csplit[0]= rp.csplit[0];
		    }
		else for (k=0; k<nc; k++) tsplit->csplit[k] = rp.csplit[k];
		}
	    }
	}
    }
/* SCCS @(#)choose_surg.c	1.5 02/25/00 */
/*
** A particular split routine, optimized for the surrogate variable
**  search.  The "goodness" of a split is the total weights of concordant
**  observations between the surrogate and the primary split.
**  Note that the CART folks use the %concordance, which factors missing
**  values into the equations somewhat differently.
**
**  y is coded as  +1=left, -1=right, 0=missing 
** 
*/

static
void choose_surg(int nodenum,    int *y,         FLOAT *x,     long *order, 
		 int ncat,    double *agreement, FLOAT *split, int *csplit,
		 double tleft,double tright,     double *adj)
    {
    int i,j;
    int agree;
    int lcount, rcount;
    int ll, lr, rr, rl;
    double llwt, lrwt, rrwt, rlwt;   /* sum of weights for each */
    int defdir;
    FLOAT lastx;
    int  *which, *left, *right;
    double *lwt, *rwt;
    double majority, total_wt;

    which = rp.which;
    left = rp.left;
    right =rp.right;
    lwt   =rp.lwt;
    rwt   =rp.rwt;

    if (ncat==0) {  /* continuous case */
	/*
	** ll = y's that go left that are also sent left by my split
	** lr = y's that go left that I send right
	** rl= y's that go right that I send to the left
	** rr= y's that go right that I send to the right
	**
	** The agreement is max(ll+rr, lr+rl), if weights were =1;
	**   actually max(llwt + rrwt, lrwt + rlwt)/ denominator
	**
	** I enforce that at least 2 obs must go each way, to avoid having an
	**  uncorrelated surrogate beat the "null" surrogate too easily
	*/
	ll = rl =0;
	llwt =0; rlwt =0;
	for (i=rp.n-1; i>=0; i--) { /*start with me sending all to the left */
	    j = order[i];
	    if (j>=0 &&  which[j]==nodenum) {
		lastx = x[i];        /*this is why I ran the loop backwards*/
		switch( y[j]) {
		    case LEFT : ll++;
			        llwt += rp.wt[j];
				break;
		    case RIGHT: rl++;
			        rlwt += rp.wt[j];
				break;
		    default   :;
		    }
		}
	    }
	lr = rr =0;
	lrwt =0; rrwt=0;
	if (llwt > rlwt) agree = llwt;
	else             agree = rlwt;

	majority   = agree;             /*worst possible agreement */
	total_wt  = llwt + rlwt;
	/*
	**  March across, moving things from the right to the left
	**    the "lastx" code is caring for ties in the x var
	*/
	for (i=0; (ll+rl)>=2; i++) {
	    j = order[i];
	    if (j >=0 &&  which[j]==nodenum) {       /* obs is in this node */
		if ((lr+rr)>=2  &&  x[i] != lastx) {
		    /* new x found, evaluate the split */
		    if ((llwt +rrwt) > agree) {
			agree = llwt + rrwt;
			csplit[0] = RIGHT;       /* < goes to the right */
			*split = (x[i] + lastx)/2;
			}
		    else if ((lrwt +rlwt) > agree) {
			agree = lrwt + rlwt;
			csplit[0] = LEFT;
			*split = (x[i] + lastx)/2;
			}
		    }

		switch (y[j]) {    /* update numbers */
		    case LEFT : ll--; lr++;
			        llwt -= rp.wt[j];
				lrwt += rp.wt[j];
				break;
		    case RIGHT: rl--; rr++;
			        rlwt -= rp.wt[j];
				rrwt += rp.wt[j];
				break;
		    default   : ;         /*ignore missing y's */
		    }
		lastx = x[i];
		}
	    }
	}

    else {     /* categorical predictor */
	for (i=0; i<ncat; i++) {
	    left[i] =0;
	    right[i]=0;
	    lwt[i] = 0;
	    rwt[i] = 0;
	    }

	/* First step:
	**  left = table(x[y goes left]), right= table(x[y goes right])
	**  lwt & rwt = weight sums
	*/
	for (i=0; i<rp.n; i++) {
	    if (which[i] == nodenum &&  order[i]>=0) {
		j = x[i] -1;
		switch( y[i]) {
		    case LEFT : left[j]++;
			        lwt[j] += rp.wt[i];
				break;
		    case RIGHT: right[j]++;
			        rwt[j] += rp.wt[i];
				break;
		    default:;
		    }
		}
	    }
	if (llwt > rlwt) agree = llwt;
	else             agree = rlwt;

	majority   = agree;             /*worst possible agreement */
	total_wt  = llwt + rlwt;

	/*
	**  Tied categories get sent to the majority, if possible
	*/
	lcount=0; rcount=0;
	llwt =0;  rrwt =0;
	for (i=0; i<ncat; i++) {
	    lcount += left[i];
	    rcount += right[i];
	    llwt += lwt[i];
	    rrwt += rwt[i];
	    }
	if (llwt > rrwt) {
	    defdir= LEFT;
	    majority= llwt;
	    }
	else {
            defdir= RIGHT;
	    majority =rrwt;
	    }
	total_wt  = llwt + rrwt;

	/* 
	** We can calculate the best split category by category--- send each
	**  individually to its better direction
	*/
	agree =0;
	for (i=0; i<ncat; i++) {
	    if (left[i]==0 && right[i]==0) csplit[i]=0;
	    else {
		if (lwt[i]< rwt[i] || (lwt[i]==rwt[i] && defdir==RIGHT)) {
		    agree+= rwt[i];
		    csplit[i] = RIGHT;
		    }
		else {
		    agree += lwt[i];
		    csplit[i] = LEFT;
		    }
		}
	    }
	}

    /*
    **  Now we have the total agreement.  Calculate the %agreement and
    **    the adjusted agreement
    **  For both, do I use the total y vector as my denominator (my
    **    preference), or only the y's for non-missing x (CART book).
    */
    if (rp.sur_agree ==0) { /* use total table */
	total_wt = tleft + tright;
	if (tleft > tright) majority=tleft;
	else                majority=tright;
	}

    *agreement = agree/ total_wt;
    majority /= total_wt;
    *adj = (*agreement -majority)/ (1-majority);
    }
/* SCCS @(#)fix_cp.c	1.2 02/08/98 */
/*
** When partition is done, each node is labeled with the complexity
**  appropriate if it were the top of the tree.  Actually, the complexity
**  should be min(me, any-node-above-me).  This routine fixes that.
*/

static
void fix_cp(struct node *me, double parent_cp)
    {
    if (me->complexity > parent_cp)  me->complexity = parent_cp;

    if (me->leftson != 0) {
	fix_cp(me->leftson, me->complexity);
	fix_cp(me->rightson,me->complexity);
	}
    }
/* SCCS @(#)free_tree.c	1.2 02/08/98 */
/*
** free up all of the memory associated with a tree
*/

static
void free_tree(struct node *node,  int freenode)
    {
    struct split *s1, *s2;

    if (node->rightson !=0) free_tree(node->rightson, 1);
    if (node->leftson  !=0) free_tree(node->leftson,  1);

    for (s1=node->surrogate; s1!=0; ){
	s2 = s1;
	s1 = s1->nextsplit;
	Free(s2);
	}
    for (s1=node->primary; s1!=0; ){
	s2 = s1;
	s1 = s1->nextsplit;
	Free(s2);
	}
    if (freenode==1) Free(node);
    }
/*   @(#)gini.c	1.11 12/13/99    
** The routines for gini-classification
*/

static int    numclass;
static double *left,     /*left branch n (weighted)*/
	      *right,
	      **ccnt;
static double *prior,
	      *aprior,   /*altered priors */
	      *freq,
              *loss;      /* loss matrix */
static int    *tsplit,
	      *countn;
static double *awt,
	      *rate;
static double (*impurity)();

static double gini_impure1(p) double p; {  return(1 - p*p); }

static double gini_impure2(p)
double p; { if (p==0) return(0.0); else return(-p*log(p)); }

static
int giniinit(int n,        double **y, int maxcat, char **error, 
	     double *parm, int *size,  int who,    double *wt)
    {
    int i, j, k;
    double temp;

    /* allocate memory  and setup losses */
    if (who==1) {
        numclass =0;   /*number of classes */
	for (i=0; i<n; i++)
	    if (*y[i] > numclass)  numclass = *y[i];

	if (parm[numclass + numclass*numclass] ==2)
		impurity = gini_impure2;
	else    impurity = gini_impure1;
 
	left = (double *) ALLOC(numclass*2, sizeof(double));
	right = left+numclass;

	tsplit= (int *) ALLOC(maxcat*2, sizeof(int));
	countn= tsplit + maxcat;

	awt = (double *) ALLOC(maxcat*2, sizeof(double));
	rate= awt +maxcat;

	if (maxcat>0) {
	    graycode_init0(maxcat);
	    ccnt    = (double **) ALLOC(numclass, sizeof(double *));
	    if (ccnt==0) {*error="Out of memory"; return(1);}
	    ccnt[0] = (double *) ALLOC(numclass*maxcat, sizeof(double));
	    if (ccnt[0]==0) {*error="Out of memory"; return(1);}
	    for (i=1; i<numclass; i++)
		ccnt[i] = ccnt[i-1] + maxcat;
	    }

	i = 3*numclass + numclass*numclass;
	prior = (double *) ALLOC(i, sizeof (double));
	if (prior==0) {*error="Out of memory"; return(1);}
	aprior = prior + numclass;
	freq   = aprior+ numclass;
	loss   = freq  + numclass;

	for (i=0; i<numclass; i++)  freq[i] =0;
	temp =0;
	for (i=0; i<n; i++) {
	    j = *y[i] -1;
	    freq[j] += wt[i];
	    temp += wt[i];   /*sum total of weights */
	    }
	for (i=0; i<numclass; i++)  freq[i] /=temp;   /*relative frequency */

	temp =0;
	for (i=0; i<numclass; i++) {
	    prior[i] = parm[i];
	    aprior[i] =0;
	    for (j=0; j<numclass; j++) {
		k = numclass*i + j;
		loss[k] = parm[numclass+k];
		temp += loss[k] * prior[i];
		aprior[i] += loss[k] * prior[i];
		}
	    }
	for (i=0; i<numclass; i++) {
	    prior[i] /= freq[i];            
	    aprior[i] /= (temp * freq[i]);  /* pi_i / n_i */
	    }
	}

    *size = 1 + numclass;
    return(0);
    }

/*
** Compute the predicted response and the classification error
**   This is R(T) (this node's contribution) in the paper.
*/
void ginidev(int n, double **y, double *value, double *risk, double *wt)
    {
    int i, j, max;
    double  temp, dev;

    dev =0;
    /*
    ** count up number in each class
    */
    for (i=0; i<numclass; i++)  freq[i]=0;
    for (i=0; i<n; i++) {
	j = y[i][0] -1;
	freq[j] += wt[i];
	}

    /*
    ** Now compute best class and its error
    */
    for (i=0; i<numclass; i++) {  /* assume class i */
	temp =0;
	for (j=0; j<numclass; j++) {
	    temp += freq[j] * loss[j*numclass +i] *prior[j];
	    }
	if (i==0 || temp < dev) {
	    max =i;
	    dev = temp;
	    }
	}

    value[0] = max +1;    /* remember: external groups start at 1 */
    for (i=0; i<numclass; i++) value[i+1] = freq[i];
    *risk  = dev;
    }


/*
** return the error for a particular point
*/
static
double ginipred(double *y, double *pred)
    {
    int i, j;
    double temp;
    i = y[0] -1;
    j = *pred -1;
    temp = prior[i]*loss[i*numclass +j];
    return(temp);
    }


/*
** The gini splitting function.  Find that split point in x such that
**  the rss within the two groups is decreased as much
**  as possible.
*/
void gini(int n,    double *y[],     FLOAT *x,     int numcat,
	  int edge, double *improve, FLOAT *split, int *csplit, double my_risk,
	  double *wt)
    { 
    int i,j,k;
    double lwt, rwt;
    int  rtot, ltot;
    int    direction, where;
    double total_ss,
	   best,
	   temp, p;
    double lmean, rmean;    /* used to decide direction */

    for (i=0; i<numclass; i++) {
	left[i] =0;
	right[i]=0;
	}
    lwt =0;  rwt=0;
    rtot=0;  ltot=0;
    for (i=0; i<n; i++) {
	j = *y[i] -1;
	rwt += aprior[j] * wt[i];    /*altered weight = prior * case_weight */
	right[j] += wt[i];
	rtot++;
	}
    total_ss =0;
    for (i=0; i<numclass; i++){
	temp = aprior[i] * right[i]/ rwt;      /* p(class=i, given node A) */
	total_ss += rwt * (*impurity)(temp);    /* p(A) * I(A) */
	}
    best =total_ss; /* total weight of right * impurity of right + 0 *0 */

    /*
    ** at this point we split into 2 disjoint paths
    */
    if (numcat >0) goto categorical;

    for (i=0;  rtot >edge; i++) {
	j = *y[i] -1;
	rwt -= aprior[j] * wt[i];
	lwt += aprior[j] * wt[i];
	rtot--;
	ltot++;
	right[j] -= wt[i];
	left[j]  += wt[i];

	if (x[i+1] != x[i] &&  (ltot>=edge)) {
	    temp =0;
            lmean =0; rmean =0;
	    for (j=0; j<numclass; j++) {
		p = aprior[j]*left[j]/lwt;    /* p(j | left) */
		temp += lwt * (*impurity)(p);      /* p(left) * I(left) */
		lmean += p*j;
                p =  aprior[j]*right[j]/rwt;   /* p(j | right) */
		temp += rwt * (*impurity)(p);      /*p(right) * I(right) */
		rmean += p*j;
	        }
	    if (temp < best) {
		best=temp;
		where =i;
		if (lmean < rmean) direction = LEFT;
		    else           direction = RIGHT;
		}
	    }
	}

    *improve =  total_ss - best;
    if (*improve > 0 ) {   /* found something */
	csplit[0] = direction;
	*split = (x[where] + x[where+1]) /2;
	}
    return;

categorical:;
    /*
    ** First collapse the data into a numclass x numcat array
    **  ccnt[i][j] = number of class i obs, category j of the predictor
    */
    for (j=0; j<numcat; j++) {
	awt[j] =0;
	countn[j]=0;
	for (i=0; i<numclass; i++)
	    ccnt[i][j] =0;
	}
    for (i=0; i<n; i++) {
	j = *y[i] -1;
	k = x[i] -1;
	awt[k] += aprior[j] * wt[i];
	countn[k]++;
	ccnt[j][k] += wt[i];
	}

    for (i=0; i<numcat; i++){ 
	if (awt[i]==0) tsplit[i] =0;
	else {
	    rate[i] = ccnt[0][i] / awt[i];   /* a scratch array */
	    tsplit[i]=RIGHT;
	    }
        }

    if (numclass==2) graycode_init2(numcat, countn, rate);
                else graycode_init1(numcat, countn);

    while((i=graycode()) < numcat) {
	/* item i changes groups */
	if (tsplit[i]==LEFT) {
	    tsplit[i]=RIGHT;
	    rwt  += awt[i];
	    lwt -= awt[i];
	    rtot += countn[i];
	    ltot -= countn[i];
	    for (j=0; j<numclass; j++) {
		right[j] += ccnt[j][i];
		left[j]  -= ccnt[j][i];
    	        }
	    }
	else {
	    tsplit[i]=LEFT;
	    rwt -= awt[i];
	    lwt += awt[i];
	    rtot -= countn[i];
	    ltot += countn[i];
	    for (j=0; j<numclass; j++) {
		right[j] -= ccnt[j][i];
		left[j]  += ccnt[j][i];
	    }
	}

	if (ltot>=edge  &&  rtot>=edge) {
	    temp =0;
	    lmean=0; rmean =0;
	    for (j=0; j<numclass; j++) {
		p = aprior[j]*left[j] /lwt;
		temp +=  lwt * (*impurity)(p);
		lmean += p*j;
                p =  aprior[j]*right[j]/rwt;       /* p(j | right) */
		temp += rwt * (*impurity)(p);      /*p(right) * I(right) */
		rmean += p*j;
	        }
	    if (temp < best) {
		best=temp;
		if (lmean < rmean)
			for (j=0; j<numcat; j++) csplit[j] = tsplit[j];
		else
			for (j=0; j<numcat; j++) csplit[j] = -tsplit[j];
	        }
	    }
        }
    *improve = total_ss - best;
    }

/* SCCS @(#)graycode.c	1.5 12/13/99 */
/*
** Walk through subsets in an ordered way.
**   For all subsets, this is the classic gray code.
**
** Graycode_init0 is called once at the very beginning with
**   the maximum number of categories.  It allocates a scratch vector.
**
** Graycode_init1 is called once for each unordered variable.
** Graycode_init2 is called once for each orderable variable,
**   the second argument is a vector that will be used to rank the variables
*/
static int *gray;
static int maxc, gsave;

static
void graycode_init0( int maxcat)
    {
    gray = (int *) ALLOC(maxcat, sizeof(int));
    }

static
void graycode_init1(int numcat, int *count)
    {
    int i;

    maxc = numcat;
    for (i=0; i<maxc; i++) {
	    if (count[i] !=0) gray[i] =1;
	    else              gray[i] =0;
	    }
    gsave = -2;
}


static
void graycode_init2(int numcat, int *count, double *val)
{
    int i,j,k;
    double temp;
    maxc = numcat;

    /* 
    **   sort categories with no members first
    **   then order by val
    */
    gray[0]=0;
    if (count[0] ==0) k=1; else k=0;
    for (i=1; i<maxc; i++) {
	if (count[i] ==0) {
	    for (j=i-1; j>=k; j--) {
		gray[j+1] = gray[j];
		val[j+1] = val[j];
	        }
	    gray[k++] =i;
	    }
	else {
	    temp = val[i];
	    for (j=i-1; j>=k && val[j]>temp; j--) {
		gray[j+1] = gray[j];
		val[j+1] = val[j];
	        }
	    val[j+1] = temp;
	    gray[j+1] =i;
	    }
        }
    gsave = k-1;
    }

/*
** Everyone starts in the right hand group
** This routine returns the next subject who needs to
**  change allegiance.
** A value of maxc means that we're done.
*/
static
int graycode() {
    int i;

    if (gsave > -2) {  /* ordered data */
	gsave++;
	if (gsave < maxc) return(gray[gsave]);
	else return (maxc);
	}

    else {
	/*
	** Form next subgroup.  We do this using the classic Gray code.
	**  The initial subset has everyone in the right group.  Each
	**  subset varies from the prior by only one member -- the
	**  following item changes groups: 1,2,1,4,1,2,1,8,1,2,1,4,1,...
	*/
	for (i=0; i<maxc; i++) {
	    if (gray[i] ==1 ) {
		gray[i] =2;
		break;
		}
	    else if (gray[i]==2) gray[i] =1;
	    }
	return(i);
	}
    }
/* SCCS @(#)insert_split.c	1.4 12/13/99  */
/*
** sort a new split into a linked list, based on its "improvement"
**
**  allocates new memory as needed
**   returns 0 if the new element isn't good enough,
**   the address of the new element otherwise
*/

static
struct split *insert_split(struct split **listhead, int ncat, 
			   double improve,          int max)
    {
    int nlist;
    struct split *s1, *s2, *s3, *s4;

    if (ncat==0) ncat=1;     /* ensure "ncat-1" below never gives a negative */
    if (*listhead ==0) {
	/* first call to a new list */
	s3 = (struct split *)CALLOC(1, sizeof(struct split)+
						(ncat-1)*sizeof(int));
	s3->nextsplit =0;
	*listhead = s3;
	return(s3);
	}

    if (max <2) {
	/* user asked for only 1 to be retained! */
	s3 = *listhead;
	if (improve <= s3->improve) return(0);
	if (ncat >1) {
	    free(s3);
	    s3 = (struct split *)CALLOC(1, sizeof(struct split)+
						(ncat-1)*sizeof(int));
	    s3->nextsplit =0;
	    *listhead = s3;
	    }
	return(s3);
	}

    /*set up --- nlist = length of list, s4=last element, s3=next to last */
    nlist=1;
    for (s4= *listhead; s4->nextsplit!=0; s4=s4->nextsplit) {
	s3 = s4;
	nlist++;
	}

    /* now set up so that the "to be added" is between s1 and s2 */
    s1 = *listhead;
    for (s2= *listhead; s2 !=0; s2= s2->nextsplit) {
	if (improve > s2->improve) break;
	s1 = s2;
	}

    if (nlist==max) {
	if (s2==0)  return(0);        /* not good enough */
	if (ncat >1) {
	    free(s4);              /*get new memory-- this chunk may be too small */
	    s4 = (struct split *)CALLOC(1, sizeof(struct split) +
						     (ncat-2)*sizeof(int));
	    }
	if (s1==s3)   s4->nextsplit =0;
	else         {s3->nextsplit =0;  s4->nextsplit =s2;}
	}
    else {
	s4 = (struct split *)CALLOC(1, sizeof(struct split) +
							(ncat-2)*sizeof(int));
	s4->nextsplit = s2;
	}
    if (s2== *listhead) *listhead     = s4;
    else                s1->nextsplit = s4;
    return(s4);
    }
/* SCCS @(#)make_cp_list.c	1.3 12/13/99 */
/*
** This routine creates the list of unique complexity parameters.
** The list is maintained in sorted order.  If two parameters are within
** "cplist_epsilon" of one another, then only the larger of them is
** retained.
**   I want the list sorted with the largest cp at the top of the list, since
** that is the order that the CP table will be printed in.
**
**    After the partition routine is done, each node is labeled with the
** complexity parameter appropriate if that node were the top of the tree.
** However, if there is a more vunerable node further up, the node in
** question will actually have the smaller complexity parameter; it will
** be removed when its parent collapses. So this routine also adjusts each
** C.P. to = minimum(my C.P., parent's C.P.).
**
**   This routine is called at the top level by rpart, after rpart has
** initialized the first member of the linked cp-list, set its number of
** splits to zero, and its risk to that for no splits at all.  This routine
** allocates and links in the rest of the cp-list.  The make_cp_table
** routine then fills in the rest of the variables in the list.
**
**  node *me;         pointer to my node structure  
**  double parent;    complexity of my parent node 
**
**   When it comes time to cross-validate, we fill in xrisk and xstd
*/

static
void make_cp_list(struct node *me, double parent, struct cptable *cptable_head)
    {
    double me_cp;
    struct cptable *cplist, *cptemp;

    if (me->complexity > parent) me->complexity = parent;
    me_cp = me->complexity;
    if (me_cp < rp.alpha) me_cp = rp.alpha;    /*table should go no lower */
    if (me->leftson != 0) {
	 make_cp_list(me->leftson, me_cp, cptable_head);
	 make_cp_list(me->rightson,me_cp, cptable_head);
	 }

    if (me_cp < parent) {  /*if not, then it can't be unique */
	for (cplist= cptable_head; cplist !=0; cplist= cplist->forward) {
	    /* am I tied? */
	    if (me_cp==0 && cplist->cp==0) return;
	    if ((fabs(cplist->cp -me_cp)/ (cplist->cp + me_cp)) < CPLIST_EPS){
		if (me_cp > cplist->cp) cplist->cp = me_cp;
		return;
		}


	    /* should I be inserted? */
	    if (me_cp > cplist->cp) break;
	    cptemp = cplist;
	    }

	/* insert new stuff after cptemp */
	cplist = (struct cptable *) CALLOC(1, sizeof(struct cptable));
	cplist->cp = me_cp;
	cplist->xrisk = 0;
	cplist->xstd  =0;
	cplist->back = cptemp;
	cplist->forward = cptemp->forward;
	if (cptemp->forward!=0) (cptemp->forward)->back = cplist;
	  else  cptable_tail = cplist;
	cptemp->forward = cplist;
	rp.num_unique_cp++;
	return;
	}
    }
/* SCCS @(#)make_cp_table.c	1.2 02/08/98 */
/*
** Given a cptable list already initialized with the unique cp's in it,
**  fill in the columns for risk and number of splits.
**
** The basic logic is : for each terminal node on the tree, start myself
**  down at the bottom of the list of complexity parameters.  For each
**  unique C.P. until my parent collapses, the node I'm in adds into that
**  line of the CP table.  So walk up the CP list, adding in, until my
**  parent would collapse; then report my position in the cp list to the
**  parent and quit.
**
**  parent: complexity of the parent node
*/

static
struct cptable *make_cp_table(struct node *me, double parent, int nsplit)
    {
    struct cptable *cplist;

    if (me->leftson) {  /* if there are splits below */
	/*
	** The 2 lines below are perhaps devious
	**  1) Since the return value depends on ones parent, both calls will
	**       return the same thing.
	**  2) I send 0 to the left to keep the current split (me) from
	**       being counted twice, once by each child.
	*/
	make_cp_table(me->leftson, me->complexity, 0);
	cplist = make_cp_table(me->rightson, me->complexity, nsplit+1);
	}
    else cplist = cptable_tail;

    while (cplist->cp < parent) {
	cplist->risk += me->risk;
	cplist->nsplit += nsplit;
	cplist = cplist->back;
	}

    return(cplist);
    }
/* SCCS @(#)mysort.c	1.6 12/13/99 */
/*
** quick sort routine : sort a vector of floats, and carry along an int
**
**  x:     vector to sort on
**  start: first element of x to sort
**  stop:  last element of x to sort
**  cvec:  a vector to carry along
*/

static
void mysort(int start, int stop, FLOAT *x, int *cvec)
 {
 int i, j, k;
 FLOAT temp, median;
 int tempd;

  while (start < stop) {
    /*
    ** first-- if the list is short, do an ordinary insertion sort
    */
    if ((stop-start)<11) {
	for (i=start+1; i<=stop; i++) {
	    temp = x[i];
	    tempd= cvec[i];
	    j=i-1;

	    while (j>=start && (x[j]>temp)) {
		x[j+1] = x[j];
		cvec[j+1] = cvec[j];
		j--;
		}
	    x[j+1] = temp;
	    cvec[j+1]  = tempd;
	    }
	return;
	}

    /*
    ** list is longer -- split it into two
    **  I use the median of 3 values as the split point
    */
    i=start;
    j=stop;
    k = (start + stop)/2;

    median = x[k];
    if (x[i] >= x[k]) {      /* one of j or k is smallest */
	if (x[j] > x[k]) {   /* k is smallest */
	    if (x[i] > x[j])  median = x[j];
	    else median= x[i];
	    }
	}
    else {
	if (x[j] < x[k]) {
	    if (x[i] > x[j]) median = x[i];
	    else median = x[j];
	    }
	}

    /* 
    **  Now actually do the partitioning 
    **   Because we must have at least one element >= median, "i"
    **   will never run over the end of the array.  Similar logic
    **   applies to j.
    ** A note on the use of "<" rather than "<=".  If a list has lots
    **   of identical elements, e.g. 80/100 are "3.5", then we will
    **   often go to the swap step with x[i]=x[j]=median.  But we will
    **   get the pointers i and j to meet approximately in the middle of
    **   the list, and that is THE important condition for speed in a
    **   quicksort.
    **   
    */
    while (i<j) {
	/*
	** top pointer down till it points at something too large
	*/
	while (x[i] < median) i++;

	/*
	** bottom pointer up until it points at something too small
	*/
	while(x[j] > median) j--;

	if (i<j) {
	    if (x[i] > x[j]) {  /* swap */
		temp = x[i];
		x[i] = x[j];
		x[j] = temp;
		tempd= cvec[i];   cvec[i] =cvec[j];  cvec[j] =tempd;
		}
	    i++; j--;
	    }
	}

    /*
    ** The while() step helps if there are lots of ties.  It will break
    **  the list into 3 parts: < median, ==median, >=median, of which only
    **  the top and bottom ones need further attention.
    ** The ">=" is needed because i may be  == to j
    */
    while (x[i] >= median && i>start) i--;
    while (x[j] <= median && j<stop ) j++;

    /*
    ** list has been split, now do a recursive call
    **   always recur on the shorter list, as this keeps the total
    **       depth of nested calls to less than log_base2(n).
    */
    if ((i-start) < (stop-j)) { /* top list is shorter */
	if ((i-start)>0) mysort(start,i, x, cvec);
	start =j; 
	}

    else {    /* bottom list is shorter */
	if ((stop -j)>0) mysort(j,stop, x, cvec);
	stop=i; 
	}
     }
  }
/* SCCS  @(#)nodesplit.c	1.4 12/13/99 */
/*
** nodesplit -- Split the node in two, and keep a count as we do of how
**  many splits are determined by each surrogate variable.
*/

static
void nodesplit(struct node *me, int nodenum)
    {
    int i,j, k;
    struct split *tsplit;
    int    var,
	   extra,
	   lastisleft,
	   someleft;
    FLOAT  psplit;
    int    *index;
    int    *which;
    long   **sorts;
    FLOAT  **xdata;

    which = rp.which;
    sorts = rp.sorts;
    xdata = rp.xdata;
    /*
    ** Walk through the variables (primary, then surrogate 1, then surr 2...)
    **   and reassign "which"
    */
    tsplit = me->primary;
    var = tsplit->var_num;
    someleft =0;
    if (rp.numcat[var] >0) {
	index = tsplit->csplit;
	for (i=0; i<rp.n; i++) {
	    if (which[i] != nodenum) continue;
	    if (sorts[var][i] <0) someleft++;
	    else switch (index[(int)xdata[var][i]-1]) {
		case LEFT:  which[i] = 2*nodenum;  break;
		case RIGHT: which[i] = 2*nodenum +1;  break;
		}
	    }
	}
    else {
	psplit = tsplit->spoint;
	extra = tsplit->csplit[0];
	for (i=0; i<rp.n; i++) {
	    j = sorts[var][i];
	    if ( j <0) {
		if (which[-(j+1)]==nodenum) someleft++;
		}
	    else if (which[j]==nodenum){
		if (xdata[var][i] < psplit)  k = extra;
		else                         k = -extra;
		if (k==LEFT) which[j] = 2*nodenum;
		else         which[j] = 2*nodenum +1;
		}
	    }
	}

    /* Now the surrogates */
    if (rp.usesurrogate==0) return;
    for (tsplit=me->surrogate;  tsplit!=0 && someleft>0;
				   tsplit=tsplit->nextsplit) {
	someleft=0;
	var = tsplit->var_num;
	if (rp.numcat[var] >0) {
	    index = tsplit->csplit;
	    for (i=0; i<rp.n; i++) {
		if (which[i]!=nodenum) continue;
		if (sorts[var][i] <0) someleft++;
		else {
		    k = xdata[var][i];    /*the value of the surrogate var */
		    /*
		    ** The need for the if stmt below may not be
		    **  obvious. The surrogate's value must not be missing,
		    **  AND there must have been at least 1 person with
		    **  both this level of the surrogate and a primary
		    **  split value somewhere in the node.  If everyone in
		    **  this node with level k of the surrogate also had a
		    **  missing value of the primary variable, then index[k-1]
		    **  will be zero.
		    */
		    if (index[k-1] !=0) {
			tsplit->count++;
			if (index[k-1] == LEFT) which[i] = 2*nodenum;
			else                    which[i] = 2*nodenum +1;
			}
		    else someleft++;
		    }
		}
	    }
	else {
	    psplit= tsplit->spoint;
	    extra = tsplit->csplit[0];
	    for (i=0; i<rp.n; i++) {
		j = sorts[var][i];
		if (j<0)  { /* missing */
		    if (which[-(j+1)]==nodenum) someleft++;
		    }
		else if (which[j]==nodenum){
		    tsplit->count++;
		    if (xdata[var][i] < psplit) k = extra;
		    else                        k = -extra;
		    if (k==LEFT)  which[j] = 2*nodenum;
		    else          which[j] = 2*nodenum +1;
		    }
		}
	    }
	}

    if (someleft>0 && rp.usesurrogate==2) {
	/* all surrogates missing, use the default */
	i = me->lastsurrogate;
	if (i !=0) {   /*50-50 splits are possible */
	    if (i < 0) lastisleft = 2*nodenum;
	    else       lastisleft = 2*nodenum +1;

	    for (i=0; i<rp.n; i++) {
		if (which[i] == nodenum) which[i] = lastisleft;
		}
	    }
	}
    }
/* SCCS  @(#)partition.c	1.5 01/06/00 */
/*
** The main workhorse of the recursive partitioning module.  When called
**   with a node, it partitions it and then calls itself to partition the
**   children it has created.
** If the node is not splittable (too few people, or complexity is too small)
**   it simply returns.  The routine may not be able to discover that the
**   complexity is too small until after the children have been partitioned,
**   so it needs to check this at the end.
*/

static
int partition(int nodenum, struct node *splitnode, double *sumrisk)
    {
    struct node *me;
    double tempcp;
    int i,j;
    double tempcp2;
    double left_risk, right_risk;
    int left_split, right_split;
    double twt;

    me = splitnode;
    if (nodenum >1) {
	j=0;
	twt =0;
	for (i=0; i<rp.n; i++)
	    if (rp.which[i] == nodenum) {
		rp.wtemp[j]   = rp.wt[i];
		rp.ytemp[j++] = rp.ydata[i];
		twt += rp.wt[i];
		}
	(*rp_eval)(j, rp.ytemp, me->response_est, &(me->risk), rp.wtemp);
	me -> num_obs = j;
	me -> sum_wt  = twt;
	tempcp = me->risk;
	if (tempcp > me->complexity)  tempcp = me->complexity;
	}
    else {
	tempcp = me->risk;
	}

    /*
    ** Can I quit now ?
    */
    if (me->num_obs < rp.min_split  ||  tempcp <= rp.alpha  || 
	nodenum > rp.maxnode) {
	me->complexity =  rp.alpha;
	me->leftson = (struct node *)0;
	me->rightson= (struct node *)0;
	*sumrisk = me->risk;
	return(0);
	}

    /*
    ** Guess I have to do the split
    */
    bsplit(me, nodenum);
    if (me->primary ==0) {
	/*
	** This is rather rare -- but I couldn't find a split worth doing
	*/
	me->complexity = rp.alpha;
	me->leftson = (struct node *)0;
	me->rightson= (struct node *)0;
	*sumrisk = me->risk;
	return(0);
	}

    if (rp.maxsur>0) (void)surrogate(me, nodenum);
    else  me->surrogate =0;
    nodesplit(me, nodenum);

    /*
    ** split the leftson
    */
    me->leftson = (struct node *)CALLOC(1, nodesize);
    (me->leftson)->complexity = tempcp - rp.alpha;
    left_split = partition(2*nodenum, me->leftson, &left_risk);

    /*
    ** Update my estimate of cp, and split the right son.
    */
    tempcp = (me->risk - left_risk) / (left_split +1);
    tempcp2 =(me->risk - (me->leftson)->risk);
    if (tempcp < tempcp2) tempcp = tempcp2;
    if (tempcp > me->complexity)  tempcp = me->complexity;

    me->rightson = (struct node *) CALLOC(1, nodesize);
    (me->rightson)->complexity = tempcp - rp.alpha;
    right_split = partition(1+2*nodenum, me->rightson, &right_risk);

    /*
    ** Now calculate my actual C.P., which depends on children nodes, and
    **  on grandchildren who do not collapse before the children.
    ** The calculation is done assuming that I am the top node of the
    **  whole tree, an assumption to be fixed up later.
    */
    tempcp = (me->risk - (left_risk + right_risk))/
		    (left_split + right_split +1);

    /* Who goes first -- minimum of tempcp, leftson, and rightson */
    if ( (me->rightson)->complexity  > (me->leftson)->complexity ) {
	if (tempcp > (me->leftson)->complexity) {
	    /* leftson collapses first */
	    left_risk = (me->leftson)->risk;
	    left_split =0;

	    tempcp = (me->risk - (left_risk + right_risk)) /
			    (left_split + right_split +1);
	    if (tempcp > (me->rightson)->complexity) {
		/* right one goes too */
		right_risk = (me->rightson)->risk;
		right_split=0;
		}
	    }
	}

    else if (tempcp > (me->rightson)->complexity) {
	/*right hand child goes first */
	right_split =0;
	right_risk = (me->rightson)->risk;

	tempcp = (me->risk - (left_risk + right_risk)) /
			(left_split + right_split +1);
	if (tempcp > (me->leftson)->complexity) {
	    /* left one goes too */
	    left_risk = (me->leftson)->risk;
	    left_split=0;
	    }
	}

    me->complexity= (me->risk - (left_risk + right_risk))/
			(left_split + right_split +1);

    if (me->complexity <= rp.alpha ) {
	/*
	** All was in vain!  This node doesn't split after all.
	*/
	free_tree(me->leftson, 1);
	free_tree(me->rightson,1);
	me->leftson = (struct node *)0;
	me->rightson= (struct node *)0;
	*sumrisk = me->risk;
	return(0);             /*return # of splits */
	}
    else {
	*sumrisk = left_risk + right_risk;
	return(left_split +right_split +1);
	}
   }
/* SCCS @(#)poisson.c	1.5 12/13/99   */
/*
**  The functions for poisson based regression
*/

#define LEFT  (-1)     /*used for the variable "extra" in nodes */
#define RIGHT  1
#define MISSING 0

static double exp_alpha,
	      exp_beta;
static double *death,
              *wtime, 
	      *rate;
static int    *countn,
	      *order,
	      *order2;
static int    which_pred;
/*
** initialize the necessary common variables for poisson fits
*/
static
int poissoninit(int n,         double *y[], int maxcat, char **error, 
		double *param, int *size,   int who,    double *wt)
    {
    int i;
    double event, time;

    /*allocate memory for scratch */
    if (who==1 && maxcat>0) {
	death = (double *)ALLOC(3*maxcat, sizeof(double));
	rate  = death + maxcat;
	wtime = rate + maxcat;
	order = (int *) ALLOC(3*maxcat, sizeof(int));
	order2= order +maxcat;
	countn= order2 + maxcat;
	}

    /* check data */
    if (who==1) {
	for (i=0; i<n; i++) {
	    if (y[i][0] <=0) {
		*error =  "Invalid time point";
		return(1);
		}
	    if (y[i][1] < 0) {
		*error = "Invalid event count";
		return(1);
		}
	    }
	}

    /* compute the overall hazard rate */
    event=0;
    time =0;
    for (i=0; i<n; i++) {
	event += y[i][1] * wt[i];
	time  += y[i][0] * wt[i];
	}

    /*
    ** Param[0] will contain the desired CV.  If is is <=0, no shrinking
    **   is desired.  The CV determines alpha, and beta is set so that
    **   the gamma prior has the correct mean.
    */
    if (param[0] <=0) {
	exp_alpha =0;
	exp_beta  =0;
	}
    else {
	exp_alpha =  1/(param[0]*param[0]);
	exp_beta =  exp_alpha /(event/time);
	}
    /*
    ** Param[1] contains the xval rule:  1=deviance, 2=square root
    */
    which_pred = param[1];
    if (param[1] !=1 && param[1] !=2) {
	*error = "Invalid error rule";
	return(1);
	}

    *size =2;
    return(0);
    }

/*
**  Compute the error of prediction
*/
double poissonpred(double *y, double *lambda) {
    double temp, dev;

    if (which_pred==1) {
	temp = y[1];
	dev  = temp - *lambda*y[0];
	if (temp>0)
	    dev += temp * log(*lambda * y[0] / temp);
	
	return(-2*dev);
	}
    else {
	/*
	** A version based on square roots, which is the
	**   variance stabilizing transform
	*/
	temp = sqrt(y[1]) - sqrt(*lambda * y[0]); /*sqrt(obs) - sqrt(exp) */
	return(temp*temp);
	}	
    }

/*
** Compute the predicted response rate (empirical Bayes) and the
**   contribution to the deviance under that rate.
** Deviance = \sum w_i[ d_i \log(d_i/ p_i) - (d_i - p_i) ]
**   where p_i = predicted # events = \lambda t_i
*/
void poissondev(int n, double **y, double *value, double *risk, double *wt)
    {
    int i;
    double  death, time;
    double lambda, dev;
    double temp;

    death =0;
    time =0;

    /*
    ** first get the overall estimate of lambda
    */
    for (i=0; i<n; i++) {
	death += y[i][1] * wt[i];
	time  += y[i][0] * wt[i];
	}
    lambda = (death + exp_alpha) / (time + exp_beta);

    dev =0;
    for (i=0; i<n; i++) {
	temp = y[i][1];
	dev -= (lambda*y[i][0] - temp) * wt[i];
	if (temp>0)
	    dev += (temp * log(lambda * y[i][0] / temp)) * wt[i];
	}

    value[0] = lambda;
    value[1] = death;
    *risk  = -2 * dev;
    }

/*
** The poisson splitting function.  Find that split point in x such that
**  the dev within the two groups is decreased as much
**  as possible.  It is not necessary to actually calculate the devs,
**  as nearly everything cancels.  The search for a split does not use the
**  Bayes estimate, for speed reasons.  
** With \hat\lambda = (\sum w_i d_i) / (\sum w_i t_i)
**  we have  deviance(total) - [ deviance(left son) + deviance(right son)]
**              =  d_l \lambda_l +  d_r \lambda_r -  d_t \lambda_t,
**   where d_l = weigthed sum of deaths for the left son, d_r = right, 
**   d_t = total, and lambda_l etc are the estimated response rates
*/
static
void poisson(int n,       double **y,      FLOAT *x,     int nclass, 
	     int edge,    double *improve, FLOAT *split,
	     int *csplit, double my_risk,  double *wt)
    {
    int i,j;
    int    left_n, right_n;
    double left_time, right_time;
    double left_d, right_d;
    double dev;      /*dev of the parent node (me) */
    double lambda1, lambda2;
    double best, temp;
    int direction;
    int where;
    int ncat;

    /*
    ** Get the total deaths and the total time
    */
    right_d =0;
    right_time =0;
    right_n =n ;
    for (i=0; i<n; i++) {
	right_d += y[i][1] * wt[i];
	right_time += y[i][0] * wt[i];
	}

    /*
    ** Compute the overall lambda and dev
    */
    lambda2 = right_d/right_time;
    if (lambda2 ==0) {
	*improve=0;         /*no deaths to split! */
	return;
	}
    dev = right_d*log(lambda2);

    /*
    ** at this point we split into 2 disjoint paths
    */
    if (nclass >0) goto categorical;

    left_time=0;
    left_d =0;
    where = -1;
    best    = dev;
    for (i=0; i < n-edge; i++) {
	left_d  += y[i][1] * wt[i];
	right_d -= y[i][1] * wt[i];
	left_time  +=y[i][0]*wt[i];
	right_time -=y[i][0]*wt[i];

	if (x[i+1] !=x[i] &&  (1+i)>=edge) {
	    lambda1 = left_d / left_time;
	    lambda2 = right_d/ right_time;
	    temp = 0;
	    if (lambda1 >0) temp += left_d * log(lambda1);
	    if (lambda2 >0) temp += right_d* log(lambda2);
	    if (temp > best) {
		best=temp;
		where =i;
		if (lambda1  < lambda2) direction = LEFT;
				else    direction = RIGHT;
		}
	    }
	}

    *improve =  -2*(dev - best) ;
    if (where >= 0 ) {   /* found something */
	csplit[0] = direction;
	*split = (x[where] + x[where+1]) /2;
	}
    return;

categorical:;
    for (i=0; i<nclass; i++) {
	wtime[i] =0;
	death[i] =0;
	countn[i]=0;
	}

    for (i=0; i<n; i++) {
	j = x[i] -1;
	countn[j]++;		        /* number per group */
	death[j]+= y[i][1] * wt[i];
	wtime[j] += y[i][0] * wt[i];     /*sum of time */
	}

    /*
    ** Rank the rates  - each is scored as the number of others that it
    **  is smaller than.  Ignore the categories which had no representatives.
    */
    ncat=0;   /* may be less than nclass if not all categories are present */
    for (i=0; i<nclass; i++) {
	order[i]=0;
	if (countn[i]>0) {
	    ncat++;
	    rate[i] = death[i]/ wtime[i];
	    for (j=i-1; j>=0; j--) {
		if (countn[j] >0) {
		    if (rate[i]>rate[j])  order[j]++;
			else              order[i]++;
		    }
		}
	    }
	}
    /*
    ** order2 will point to the largest, second largest, etc
    */
    for (i=0; i<nclass; i++) {
	if (countn[i]>0) order2[order[i]]=i;
	}

    /*
    ** Now find the split that we want
    **    starting with everyone in the right hand group
    */
    left_n =0;
    left_d =0;
    left_time=0;
    best =dev;
    where =0;
    for (i=0; i<ncat-1; i++){
	j = order2[i];
	left_n += countn[j];
	right_n-= countn[j];
	left_time += wtime[j];
	right_time-= wtime[j];
	left_d  += death[j];
	right_d -= death[j];
	if (left_n>=edge  &&  right_n>=edge) {
	    lambda1 = left_d / left_time;
	    lambda2 = right_d/ right_time;
	    temp = 0;
	    if (lambda1 >0) temp += left_d * log(lambda1);
	    if (lambda2 >0) temp += right_d* log(lambda2);
	    if (temp > best) {
		best=temp;
		where =i;
		if (lambda1  < lambda2) direction = LEFT;
				else    direction = RIGHT;
		}
	    }
	}

    *improve =  -2*(dev - best) ;

    /* (if improve=0, csplit will never be looked at by the calling routine)*/
    for (i=0; i<nclass; i++)      csplit[i]=0;
    for (i=0; i<=where; i++)  csplit[order2[i]] = direction ;
    for (   ; i<ncat; i++)    csplit[order2[i]] =  -direction;

    }
/*  SCCS @(#)pred_rpart.c	1.5 02/08/98 
**
**  Do rpart predictions given the matrix form of the tree.
**
**  Input
**      dimx        : # of rows and columns in the new data
**      nnode       : # of nodes in the tree
**      nsplit      : # of split structures
**      dimc        : dimension of the categorical splits matrix
**      nnum        : node number for each row of 'nodes'
**      nodes       : matrix of node info
**                      row 0= count, 1=index of primary, 2=#competitors,
**                          3= number of surrogates
**      vnum        : variable number of each split
**      split       : matrix of split info
**                  :   row 0=useage count, 1= #categories if >1, otherwise
**                         the split parity, 2= utility, 3= index to csplit
**                         or numeric split point
**      csplit      : matrix of categorical split info
**      usesur      : at what level to use surrogates
**      xdata       : the new data
**      xmiss       : shows missings in the new data
**
**  Output
**      where       : the "final" row in nodes for each observation
*/

void pred_rpart(long *dimx,	long *nnode, 	long *nsplit, 	long *dimc, 
		long *nnum,  	long *nodes2,   long *vnum,     double *split2,
		long *csplit2,  long *usesur,   double *xdata2, 
		long *xmiss2,   long *where)
    {
    int i,j;
    int n;
    int ncat;
    int node, nspl, var, dir;
    int lcount, rcount;
    int npos;
    double temp;
    long   *nodes[4];
    double *split[4];
    long   **csplit,
	   **xmiss;
    double **xdata;

    n = dimx[0];
    for (i=0; i<4; i++) {
	nodes[i] = &(nodes2[*nnode *i]);
	split[i] = &(split2[*nsplit*i]);
	}

    if (dimc[1] > 0) {
	csplit = (long **)  ALLOC((int)dimc[1], sizeof(int*));
	for (i=0; i<dimc[1]; i++)  csplit[i] = &(csplit2[i * dimc[0]]);
	}    
    xmiss =  (long **)  ALLOC((int)dimx[1], sizeof(int*));
    xdata = (double **) ALLOC((int)dimx[1], sizeof(double*));
    for (i=0; i<dimx[1]; i++) {
	xmiss[i] = &(xmiss2[i * dimx[0]]);
	xdata[i] = &(xdata2[i * dimx[0]]);
	}

    for (i=0; i<n; i++) {
	node =1;   /*current node of the tree */
next:   for (npos=0; nnum[npos]!=node; npos++);  /*position of the node */
	/* walk down the tree */
	nspl = nodes[3][npos] -1;  /*index of primary split */
	if (nspl >=0) {            /* not a leaf node */
	    var  = vnum[nspl] -1;
	    if (xmiss[var][i]==0) {     /* primary var not missing */
		ncat = split[1][nspl];
		temp = split[3][nspl];
		if (ncat >=2) dir = csplit[(int)xdata[var][i] -1][(int)temp-1];
		else if (xdata[var][i] < temp) dir=ncat;
		     else                      dir= -ncat;
		if (dir!=0) {
		    if (dir== -1) node = 2*node;
		    else          node = 2*node +1;
		    goto next;
		    }
		}

	    if (*usesur >0 ) {
		for (j=0; j<nodes[2][npos]; j++) {
		    nspl = nodes[1][npos] + nodes[3][npos] + j;
		    var  = vnum[nspl] -1;
		    if (xmiss[var][i]==0) {     /* surrogate not missing */
			ncat = split[1][nspl];
			temp = split[3][nspl];
			if (ncat >=2) dir = csplit[(int)xdata[var][i] -1][(int)temp-1];
			else if (xdata[var][i] < temp) dir=ncat;
			     else                      dir= -ncat;
			if (dir!=0) {
			    if (dir== -1) node = 2*node;
			    else          node = 2*node +1;
			    goto next;
			    }
			}
		    }
		}

	    if (*usesur >1) { /* go with the majority */
		for (j=0; nnum[j]!= (2*node); j++);
		    lcount = nodes[0][j];
		for (j=0; nnum[j]!= (1+ 2*node); j++);
		    rcount = nodes[0][j];
		if (lcount != rcount) {
		    if (lcount > rcount) node = 2*node;
		    else                 node = 2*node +1;
		    goto next;
		    }
		}
	    }
	where[i] = npos +1;
	}
    }
/* SCCS @(#)rpart.c	1.8 01/06/00 */
/*
** The main entry point for recursive partitioning routines.
**
** Input variables:
**      n       = # of observations
**      nvarx   = # of columns in xmat
**      ncat    = # categories for each var, 0 for continuous variables.
**      method  = 1 - anova
**                2 - exponential survival
**      maxpri  = max number of primary variables to retain (must be >0)
**      parms   = extra parameters for the split function, e.g. poissoninit
**      ymat    = matrix or vector of response variables
**      xmat    = matrix of continuous variables
**      missmat = matrix that indicates missing x's.  1=missing.
**      cptable  = a pointer to the root of the complexity parameter table
**      tree     = a pointer to the root of the tree
**      error    = a pointer to an error message buffer
**      xvals    = number of cross-validations to do
**      xgrp     = indices for the cross-validations
**      wt       = vector of case weights
**      opt      = options, in the order of rpart.control()
**
** Returned variables
**      error    = text of the error message
**      which    = final node for each observation
**
** Return value: 0 if all was well, 1 for error
**
*/

static
int rpart(int n,         int nvarx,      long *ncat,     int method, 
          int  maxpri,   double *parms,  double *ymat,   FLOAT *xmat,
          long *missmat, struct cptable *cptable,
	  struct node **tree,            char **error,   int *which,
	  int xvals,     long *x_grp,    double *wt,     double *opt) {
    int i,j,k;
    int maxcat;
    double temp;

    /*
    ** initialize the splitting functions from the function table
    */
    if (method <= NUM_METHODS) {
	i = method -1;
	rp_init   = func_table[i].init_split;
	rp_choose = func_table[i].choose_split;
	rp_eval   = func_table[i].eval;
	rp_error  = func_table[i].error;
	rp.num_y  = func_table[i].num_y;
	}
    else {
	*error = "Invalid value for 'method'";
	return(1);
	}

    /*
    ** set some other parameters
    */
    rp.min_node =  opt[1];
    rp.min_split = opt[0];
    rp.complex   = opt[2];
    rp.maxsur = opt[4];
    rp.usesurrogate = opt[5];
    rp.sur_agree = opt[6];
    rp.maxnode  = pow((double)2.0, opt[8]) -1;
    rp.nvar = nvarx;
    rp.numcat = ncat;
    rp.maxpri = maxpri;
    if (maxpri <1) rp.maxpri =1;
    rp.n = n;
    rp.which = which;
    rp.wt    = wt;

    /*
    ** create the "ragged array" pointers to the matrix
    **   x and missmat are in column major order
    **   y is in row major order
    */
    rp.xdata = (FLOAT **) ALLOC(nvarx, sizeof(FLOAT *));
    for (i=0; i<nvarx; i++) {
	rp.xdata[i] = &(xmat[i*n]);
	}
    rp.ydata = (double **) ALLOC(n, sizeof(double *));
    for (i=0; i<n; i++)  rp.ydata[i] = &(ymat[i*rp.num_y]);

    /*
    ** allocate some scratch
    */
    rp.tempvec = (int *)ALLOC(n, sizeof(int));
    rp.xtemp = (FLOAT *)ALLOC(n, sizeof(FLOAT));
    rp.ytemp = (double **)ALLOC(n, sizeof(double *));
    rp.wtemp = (double *)ALLOC(n, sizeof(double));

    /*
    ** create a matrix of sort indices, one for each continuous variable
    **   This sort is "once and for all".  The result is stored on top
    **   of the 'missmat' array.
    ** I don't have to sort the categoricals.
    */
    rp.sorts  = (long**) ALLOC(nvarx, sizeof(long *));
    maxcat=0;
    for (i=0; i<nvarx; i++) {
	rp.sorts[i] = &(missmat[i*n]);
	for (k=0; k<n; k++) {
	    if (rp.sorts[i][k]==1) {
		rp.tempvec[k] = -(k+1);
		rp.xdata[i][k]=0;   /*weird numerics might destroy 'sort'*/
		}
	    else                   rp.tempvec[k] =  k;
	    }
	if (ncat[i]==0)  mysort(0, n-1, rp.xdata[i], rp.tempvec);
	else if (ncat[i] > maxcat)  maxcat = ncat[i];
	for (k=0; k<n; k++) rp.sorts[i][k] = rp.tempvec[k];
	}

    /*
    ** And now the last of my scratch space
    */
    if (maxcat >0) {
	rp.csplit = (int *) ALLOC(3*maxcat, sizeof(int));
	rp.lwt    = (double *) ALLOC(2*maxcat, sizeof(double));
	rp.left = rp.csplit + maxcat;
	rp.right= rp.left   + maxcat;
	rp.rwt  = rp.lwt    + maxcat;
	}
    else rp.csplit = (int *)ALLOC(1, sizeof(int));

    /*
    ** initialize the top node of the tree
    */
    temp =0;
    for (i=0; i<n; i++) {
	which[i] =1;
	temp += wt[i];
	}
    i = rp_init(n, rp.ydata, maxcat, error, parms, &rp.num_resp, 1, wt);
    nodesize = sizeof(struct node) + (rp.num_resp-2)*sizeof(double);
    *tree = (struct node *) CALLOC(1, nodesize);
    (*tree)->num_obs = n;
    (*tree)->sum_wt  = temp;
    if (i>0) return(i);

    (*rp_eval)(n, rp.ydata, (*tree)->response_est, &((*tree)->risk), wt);
    (*tree)->complexity = (*tree)->risk;
    rp.alpha = rp.complex * (*tree)->risk;

    /*
    ** Do the basic tree
    */
    partition(1, (*tree), &temp);
    if ((*tree)->rightson ==0) {
	*error = "No splits could be created";
	return(1);
	}

    cptable->cp = (*tree)->complexity;
    cptable->risk = (*tree)->risk;
    cptable->nsplit = 0;
    cptable->forward =0;
    cptable->xrisk =0;
    cptable->xstd =0;
    rp.num_unique_cp =1;
    make_cp_list((*tree), (*tree)->complexity, cptable);
    make_cp_table((*tree), (*tree)->complexity, 0);

    if (xvals >1) xval(xvals, cptable, x_grp, maxcat, error, parms);
    /*
    ** all done
    */
    return(0);
    }
/* SCCS @(#)rpartexp.c	1.1 12/13/99  */
/*
**  rescale an Survival time so that it is essentially exponential
**   each interval between deaths has the same number of person-years
**
**	n	number of observations
**	y	survival object, sorted by death times
**	wt      vector of weights, sorted
** output
**      newy    rescaled time vector
** scratch
**      wtemp(n)
*/
void rpartexp(long *n2, double *y, double *wt, double *newy, double *wtemp) {
    int n;
    double *start, *stop, *event;
    int i, j;
    double tsum, dsum;     /*weighted sums of times and deaths */
    double time, ltime, rtime;
    double temp;
    double psum, scale;
    int last;

    n = *n2;
    stop = y;
    event= y+n;

    temp =0;
    for (i=n-1; i>=0; i--) {   
	temp += wt[i];
	wtemp[i] = temp;   /* sum of weights */
	}
	
    last = 0;
    ltime =0;   /* last event time */
    rtime =0;   /* rescaled time, cumulative */
    while (last < n) {
	/*
	** look ahead to find the next death
	*/
	psum =0;
	for (i=last; i<n & event[i]==0; i++) {
	    psum += wt[i]*(stop[i]-ltime);    /*partial intervals */
	    }

	/*
	** Found it (or the end of the data)
	*/
	if (i>n) {  /* no more deaths */
	    for (i=last; i<n; i++) newy[i] = rtime;
	    last=n;
	    }
	else { /* rescale this interval */
	    /*
	    ** count up the sum of the weighted deaths
	    */
	    dsum=0;
	    time= stop[i];
	    for (; i<n && event[i]==1 && stop[i]==time; i++) {
		dsum += wt[i];  /*tied deaths */
		}
	    
	    tsum = (wtemp[i] + dsum)*(time-ltime) + psum;
	    scale = dsum/tsum;   /* scaling factor */
	    
	    for (j=last; j<i; j++) 
		newy[j] = rtime + (stop[j]-ltime)*scale;
	    rtime += (time-ltime)*scale;
	    last =i;
	    ltime = time;
	    }
	}
    }
	    
/* SCCS @(#)rpcountup.c	1.4 02/08/98 */
/*
** count up the number of nodes and splits in the final result
**
**  Gather the counts for myself, add in those of my children, and
**    pass the total back to my parent
*/

static
void rpcountup(struct node *me, long *nnode, long *nsplit, int *ncat)
    {
    long  node2, split2;
    int   cat2;
    int   i,j, k;
    struct split *ss;

    if (me->complexity <=rp.alpha || me->leftson==0) { /*no kids */
	*nnode=1;
	*nsplit=0;
	*ncat =0;
	}
    else {
	i=0; j=0; k=0;
	for (ss= me->primary;   ss!=0; ss = ss->nextsplit) {
	    i++;
	    if (rp.numcat[ss->var_num] >0) k++;
	    }
	for (ss= me->surrogate; ss!=0; ss = ss->nextsplit) {
	    j++;
	    if (rp.numcat[ss->var_num] >0) k++;
	    }

	rpcountup(me->leftson, nnode,  nsplit,  ncat);
	rpcountup(me->rightson,&node2, &split2, &cat2);
	*nnode += 1 + node2;
	*nsplit += i + j + split2;
	*ncat   += k + cat2;
	}
    }
/* SCCS @(#)rplabel.c	1.6 02/08/98 */
/*
** Called by S to make nice "labels" for the split points
**
**  csplit is a matrix with maxcat rows and ncat columns
**  splits has (nsplit-1)/2 columns, 2 rows
**
**  cutleft and cutright are the output
*/
static char *strdup();

void rplabel(long *nsplit,   long *index,   double *splits, 
             long *ncat,     long *csplit,  char   **cutleft, char **cutright)
    {
    int i,j, k;
    int  ii, jj;
    int  nn, nc, ccol;
    char buf[1000];

    nn = (*nsplit-1) /2;    /*leaves = splits +1;  *nsplit = leaves + splits*/
    nc = *ncat;
    j=0;
    for (i=0; i< *nsplit; i++) {
	if (index[i]==0) continue;

	k = splits[j];
	if (k >=2)  {  /* categorical */
	    ccol = splits[j + nn] -1;
	    buf[0] = ':';
	    ii =1;
	    for (jj=0; jj< k; jj++)
		if      (csplit[jj*nc +ccol] == LEFT)  buf[ii++] = 'a' + jj;
	    buf[ii] = '\0';
	    cutleft[i] = strdup(buf);

	    buf[0] = ':';
	    ii =1;
	    for (jj=0; jj< k; jj++)
		if      (csplit[jj*nc + ccol] == RIGHT)  buf[ii++] = 'a' + jj;
	    buf[ii] = '\0';
	    cutright[i] = strdup(buf);
	    }

	else {  /* continuous */
	    if (k == LEFT) {
		sprintf(buf, "<%.6g", splits[nn +j]);
		cutleft[i] = strdup(buf);
		sprintf(buf, ">%.6g", splits[nn +j]);
		cutright[i] = strdup(buf);
		}
	    else {
		sprintf(buf, ">%.6g", splits[nn +j]);
		cutleft[i] = strdup(buf);
		sprintf(buf, "<%.6g", splits[nn +j]);
		cutright[i] = strdup(buf);
		}
	    }
	j++;
	}
    }

static char *strdup(s)
char *s;
{
	char *strcpy(), *malloc();
	size_t strlen();
	return(strcpy(malloc(strlen(s)+1), s));
}
/* SCCS @(#)rpmatrix.c	1.6 12/13/99 */
/*
**  For S's usage, convert the linked list data into matrix form
*/

static
void rpmatrix(struct node *me,  long *nodecount,   long *splitcount, 
	      long *catcount,   long *numcat,      double **dsplit,
	      long **isplit,    long **csplit,     double **dnode, 
	      long **inode,     int id)
    {
    /*
    ** dsplit  0: improvement
    **         1: split point if continuous; index into csplit if not
    **         2: surrogate: adjusted agreement,  primary: nothing
    ** isplit  0: variable #
    **         1: count
    **         2: if continuous: direction -1=left, 1=right
    **            if categorical: # of categories
    ** csplit[i]: -1=left, 0=missing category, 1=right
    ** dnode   0: risk
    **         1: complexity threshold
    **         2: sum of weights
    **         3, ...: response estimate
    ** inode   0: node number
    **         1: index of the first primary, in the split list
    **         2: #primary    ==0 if this is a terminal node
    **         3: #surrogates
    **         4: # observations
    **         5: # obs for which this is the final resting place
    */

    int i,j, k;
    struct split *spl;
    static int scnt, ncnt, ccnt;
    static double cp_scale;

    if (id==1) {
	cp_scale = 1/ me->risk;
	}
    scnt = *splitcount;
    ncnt = *nodecount;
    ccnt = *catcount;
    dnode[0][ncnt] = me->risk;
    dnode[1][ncnt] = me->complexity * cp_scale;
    dnode[2][ncnt] = me->sum_wt;
    for (i=0; i<rp.num_resp; i++) dnode[3+i][ncnt] = me->response_est[i];
    inode[0][ncnt] = id;
    inode[4][ncnt] = me->num_obs;

    if (me->complexity <=rp.alpha || me->leftson==0) { /*no kids */
	inode[1][ncnt] = 0;
	inode[2][ncnt] = 0;
	inode[3][ncnt] = 0;
	inode[5][ncnt] = me->num_obs;
	*nodecount = ncnt+1;
	}
    else {
	inode[1][ncnt] = scnt +1;    /*S has 1 based, not 0 based subscripts */

	i=0;
	for (spl = me->primary; spl!=0; spl = spl->nextsplit) {
	    i++;
	    j = spl->var_num;
	    dsplit[0][scnt] = spl->improve;
	    if (numcat[j] ==0) {
		dsplit[1][scnt] = spl->spoint;
		isplit[2][scnt] = spl->csplit[0];
		}
	    else {
		dsplit[1][scnt] = ccnt+1;
		isplit[2][scnt] = numcat[j];
		for (k=0; k<numcat[j]; k++) csplit[k][ccnt] = spl->csplit[k];
		ccnt++;
		}
	    isplit[0][scnt] = j +1;      /* use "1" based subscripts */
	    isplit[1][scnt] = spl->count;
	    scnt++;
	    }
	inode[2][ncnt] = i;

	i=0;
	for (spl=me->surrogate; spl!=0; spl = spl->nextsplit) {
	    i++;
	    j = spl->var_num;
	    dsplit[0][scnt] = spl->improve;
	    dsplit[2][scnt] = spl->adj;
	    if (numcat[j] ==0) {
		dsplit[1][scnt] = spl->spoint;
		isplit[2][scnt] = spl->csplit[0];
		}
	    else {
		dsplit[1][scnt] = ccnt+1;
		isplit[2][scnt] = numcat[j];
		for (k=0; k<numcat[j]; k++) csplit[k][ccnt] = spl->csplit[k];
		ccnt++;
		}
	    isplit[0][scnt] = j +1;
	    isplit[1][scnt] = spl->count;
	    scnt++;
	    }
	inode[3][ncnt] = i;
	inode[5][ncnt] = me->num_obs -
			   ((me->leftson)->num_obs + (me->rightson)->num_obs);

	ncnt++;
	*nodecount = ncnt;
	*splitcount= scnt;
	*catcount  = ccnt;

	rpmatrix(me->leftson, nodecount, splitcount, catcount, numcat,
		    dsplit, isplit, csplit, dnode, inode, 2*id);
	rpmatrix(me->rightson,nodecount, splitcount, catcount, numcat,
		    dsplit, isplit, csplit, dnode, inode, 2*id +1);
	}
    }
/* SCCS @(#)rundown.c	1.4 02/08/98 */
/*
** Run an observation down the tree, and return the prediction error,
**    for several CP values at once.
*/

static
void rundown(struct node *tree,  int obs,     double *cp, 
	     double *xpred,      double *xtemp)
    {
    int i;
    struct node *otree;

    /*
    ** Now, repeat the following: for the cp of interest, run down the tree
    **   until I find a node with smaller complexity.  The parent node will
    **   not have collapsed, but this split will have, so this is my
    **   predictor.
    */
    otree = tree;
    for (i=0; i<rp.num_unique_cp; i++) {
	while ( cp[i] < tree->complexity) {
	    tree = branch(tree, obs);
	    if (tree ==0) goto oops;
	    otree = tree;
	    }
	xpred[i] =  tree->response_est[0];
	xtemp[i] = (*rp_error)(rp.ydata[obs], tree->response_est);
	}

    return;

oops:;
    if (rp.usesurrogate <2) { /*must have hit a missing value */
	for (; i<rp.num_unique_cp; i++)
	    xpred[i] =  otree->response_est[0];
	    xtemp[i] = (*rp_error)(rp.ydata[obs], otree->response_est);
	return;
	}
    /*
    ** I never really expect to get to this code.  It can only happen if
    **  the last cp on my list is smaller than the terminal cp of the
    **  xval tree just built.  This is impossible (I think).  But just in
    **  case I put a message here.
    */
    fprintf(stderr, "Warning message--see rundown.c\n");
    }
/* SCCS @(#)rundown2.c	@(#)rundown2.c	1.5 12/13/99 */
/*
** Run an observation down the tree, and return the predicted value,
**    for several CP values at once.
** (A subset of rundown.c, which also returns the prediction error).
*/

static
void rundown2(struct node *tree, int obs, double *cp, double *xpred)
    {
    int i;
    struct node *otree;

    /*
    ** Now, repeat the following: for the cp of interest, run down the tree
    **   until I find a node with smaller complexity.  The parent node will
    **   not have collapsed, but this split will have, so this is my
    **   predictor.
    */
    otree = tree;
    for (i=0; i<rp.num_unique_cp; i++) {
	while ( cp[i] < tree->complexity) {
	    tree = branch(tree, obs);
	    if (tree ==0) goto oops;
	    otree = tree;
	    }
	xpred[i] =  tree->response_est[0];
	}

    return;

oops:;
    if (rp.usesurrogate <2) { /*must have hit a missing value */
	for (; i<rp.num_unique_cp; i++)
	    xpred[i] = otree->response_est[0];
	return;
	}
    /*
    ** I never really expect to get to this code.  It can only happen if
    **  the last cp on my list is smaller than the terminal cp of the
    **  xval tree just built.  This is impossible (I think).  But just in
    **  case I put a message here.
    */
    fprintf(stderr, "Warning message--see rundown2.c\n");
    }
/* SCCS @(#)s_to_rp.c	1.14 01/06/00 */
/*
** An S interface to the the recursive partitioning routines.
*/

static struct cptable cptab;
static struct node *tree;
static int *savewhich;

void s_to_rp(long *n, 	  long *nvarx, 	 long *ncat, 	long *method, 
	     double *opt, double *parms, long *xvals,   long *x_grp,
	     double *y,   FLOAT *xmat,   long *missmat, char **error,
	     double *wt)
    {
    int itemp;
    int maxpri;
    int rval;      /* return value */
    savewhich = (int *) CALLOC((int)*n, sizeof(int));
    /*
    **  The opt string is in the order of control.rpart()
    **    minsplit, minbucket, cp, maxcomptete, maxsurrogate, usesurrogate,
    **    and xval
    */
    maxpri = opt[3] +1;
    rval = rpart( (int)*n,    (int)*nvarx,   ncat,        (int)*method,
	           maxpri,      parms,       y,            xmat,        
		  missmat,     &cptab,      &tree,        &(error[0]), 
		  savewhich,  (int)*xvals,  x_grp,         wt,         
		  opt);
    /*
    ** count up the number of nodes, splits, categorical splits, and cp's
    */
    rpcountup(tree, n, nvarx, &itemp);
    ncat[0] = itemp;
    *method = rp.num_unique_cp;
    if (rval==1) *n= -1;   /* signal an error */
    }

/*
** The routine above returns the sizes of the objects, and saves the lists
**   (the list heads are static).  S then calls again with appropriately
**   sized arrays to this routine. This stuffs the arrays and frees the memory
*/
void s_to_rp2(long *n,         long *nsplit,    long *nnode,     long *ncat, 
	      long *numcat,    long *maxcat,    long *xvals,     long *which, 
	      double *cptable, double *dsplit,  long *isplit,    long *csplit,
	      double *dnode,   long *inode)
    {
    int i;
    int  nodenum, j;
    struct cptable *cp, *cp2;
    double **ddnode  , *ddsplit[3];
    long   *iinode[6], *iisplit[3];
    long   **ccsplit;
    double scale;
    /*
    ** create the "ragged array" pointers to the matrices
    */
    ddnode = (double **) ALLOC(3+rp.num_resp, sizeof(double *));
    for (i=0; i<(3+rp.num_resp); i++) {
	ddnode[i] = dnode;  dnode  += *nnode;
	}
    for (i=0; i<3; i++) {
	ddsplit[i]= dsplit; dsplit += *nsplit;
	}
    for (i=0; i<6; i++) {
	iinode[i] = inode;  inode  += *nnode;
	}
    for (i=0; i<3; i++) {
	iisplit[i]= isplit; isplit += *nsplit;
	}

    /* I don't understand this next line.  Even if I don't need ccsplit
    ** (maxcat=0), not allocating it makes S memory fault.  Not that
    **  4 extra bytes is any big deal....
    */
    if (*maxcat==0) i=1; else i = *maxcat;
    ccsplit = (long **)CALLOC(i, sizeof(long *));
    for (i=0; i<*maxcat; i++) {
	ccsplit[i] = csplit;   csplit += *ncat;
	}

    /* retrieve the complexity table */
    scale = 1/tree->risk;
    i=0;
    for (cp = &cptab; cp !=0; cp= cp->forward) {
	cptable[i++] = cp->cp * scale;
	cptable[i++] = cp->nsplit;
	cptable[i++] = cp->risk * scale;
	if (*xvals >1) {
	    cptable[i++] = cp->xrisk*scale;
	    cptable[i++] = cp->xstd *scale;
	    }
	}

    /* Now get the tree */
    *nnode=0; *nsplit=0; *ncat=0;   /*array starting points */
    rpmatrix(tree, nnode, nsplit, ncat, numcat,
		    ddsplit, iisplit, ccsplit, ddnode, iinode, 1);

    /*
    ** Now fix up the 'which' array
    **   It would be a simple S match(), except that nodes sometimes get cut
    */
    for (i=0; i<*n; i++) {
	nodenum = savewhich[i];
	do {
	    for (j=0; j< *nnode; j++)
		if (iinode[0][j] == nodenum) {
		    which[i] = j+1;
		    break;
		    }
	    nodenum /=2;
	    }  while (j >= *nnode);
	}

    /*
    ** restore the memory
    **  since the root was not calloced, I have to not free it (second arg
    **  of free_tree).
    */
    free_tree(tree, 0);
    for (cp=cptab.forward; cp!=0; ) {
	cp2 = cp->forward;
	Free(cp);
	cp = cp2;
	}
    Free(ccsplit);
    Free(savewhich);
    }

/*
**  SCCS  @(#)s_xpred.c	1.12 02/15/00   
** An S interface to "cross validated predictions"
**    99% of this routine is a copy of s_to_rp and rpart.c
*/

void s_xpred(long *sn, 	   long *nvarx,   long *ncat,    long *method, 
	     double *opt,  double *parms, long *xvals,   long *x_grp,
	     double *ymat, FLOAT *xmat,   long *missmat, double *predict,
	     long *ncp,    double *cp,    char **error,  double *wt)
    {
    int i,j,k;
    int maxcat;
    double temp;
    int n, nvar;
    int maxpri;
    struct node *xtree;
    double old_wt, total_wt;

    /*
    ** initialize the splitting functions from the function table
    */
    if (*method <= NUM_METHODS) {
	i = *method -1;
	rp_init   = func_table[i].init_split;
	rp_choose = func_table[i].choose_split;
	rp_eval   = func_table[i].eval;
	rp_error  = func_table[i].error;
	rp.num_y  = func_table[i].num_y;
	}
    else {
	*error = "Invalid value for 'method'";
	*sn = -1; 
	return;
	}

    /*
    ** Set other parameters
    **
    **  The opt string is in the order of control.rpart()
    **    minsplit, minbucket, cp, maxcomptete, maxsurrogate, usesurrogate,
    **    and xval
    */
    n = *sn;
    nvar = *nvarx;
    rp.nvar =  nvar;
    rp.numcat = ncat;
    rp.n = n;
    rp.num_unique_cp = *ncp;
    rp.wt = wt;

    rp.min_node =  opt[1];
    rp.min_split = opt[0];
    rp.complex   = opt[2];
    maxpri       = opt[3] +1;
    rp.maxpri = maxpri;
    if (maxpri <1) rp.maxpri =1;
    rp.maxsur = opt[4];
    rp.usesurrogate = opt[5];
    rp.sur_agree = opt[6];
    rp.maxnode  = pow((double)2.0, opt[8]) -1;

    /*
    ** create the "ragged array" pointers to the matrix
    **   x and missmat are in column major order
    **   y is in row major order
    */
    rp.xdata = (FLOAT **) ALLOC(nvar, sizeof(FLOAT *));
    for (i=0; i<nvar; i++) {
	rp.xdata[i] = &(xmat[i*n]);
	}
    rp.ydata = (double **) ALLOC(n, sizeof(double *));
    for (i=0; i<n; i++)  rp.ydata[i] = &(ymat[i*rp.num_y]);

    /*
    ** allocate some scratch
    */
    rp.tempvec = (int *)ALLOC(2*n, sizeof(int));
    rp.which   = rp.tempvec +n;
    rp.xtemp = (FLOAT *)ALLOC(n, sizeof(FLOAT));
    rp.ytemp = (double **)ALLOC(n, sizeof(double *));
    rp.wtemp = (double *)ALLOC(n, sizeof(double));

    /*
    ** create a matrix of sort indices, one for each continuous variable
    **   This sort is "once and for all".  The result is stored on top
    **   of the 'missmat' array.
    ** I don't have to sort the categoricals.
    */
    rp.sorts  = (long**) ALLOC(nvar, sizeof(long *));
    maxcat=0;
    for (i=0; i<nvar; i++) {
	rp.sorts[i] = &(missmat[i*n]); 
	for (k=0; k<n; k++) {
	    if (rp.sorts[i][k]==1) {
		rp.tempvec[k] = -(k+1);
		rp.xdata[i][k] =0;     /* avoid weird numerics in S's NA */
	        }
	    else                   rp.tempvec[k] =  k;
	    }
	if (ncat[i]==0)  mysort(0, n-1, rp.xdata[i], rp.tempvec); 
	else if (ncat[i] > maxcat)  maxcat = ncat[i];
        for (k=0; k<n; k++) rp.sorts[i][k] = rp.tempvec[k];
	}

    /*
    ** And now the last of my scratch space
    */
    if (maxcat >0) {
	rp.csplit = (int *) ALLOC(3*maxcat, sizeof(int));
	rp.left = rp.csplit + maxcat;
	rp.right= rp.left   + maxcat;
        rp.lwt    = (double *) ALLOC(2*maxcat, sizeof(double));
        rp.rwt  = rp.lwt    + maxcat;
	}
    else rp.csplit = (int *)ALLOC(1, sizeof(int));

    (*rp_init)(n, rp.ydata, maxcat, error, parms, &rp.num_resp, 1, rp.wt);
    nodesize = sizeof(struct node) + (rp.num_resp-2)*sizeof(double);

    /*
    ** I need the risk of the full tree, to scale alpha
    */
    xtree = (struct node *) ALLOC(1, nodesize);
    (*rp_eval)(n, rp.ydata, xtree->response_est, &(xtree->risk), rp.wt);
    rp.alpha = rp.complex * (xtree)->risk;

    /*
    ** do the validations
    */
    total_wt =0;
    for (i=0; i<rp.n; i++) total_wt += rp.wt[i];
    old_wt = total_wt;

    for (i=0; i< *xvals; i++) {
	/*
	** mark the "leave out" data as ficticious node 0
	*/
	k=0;
	temp =0;
	for (j=0; j<rp.n; j++) {
	    if (x_grp[j]==(i+1)) {
		rp.which[j] =0;
		}
	    else {
		rp.which[j] =1;
		rp.ytemp[k] = rp.ydata[j];
		rp.wtemp[k] = rp.wt[j];
		k++;
		temp += rp.wt[j];
		}
	    }

	/* rescale the cp */
	for (j=0; j<rp.num_unique_cp; j++) cp[j] *= temp/old_wt;
	rp.alpha *= temp/old_wt;
	old_wt = temp;

	/*
	** partition the new tree
	*/
	xtree = (struct node *) calloc(1, nodesize);
	xtree->num_obs = k;
	(*rp_init)(k,rp.ytemp, maxcat, error, parms, &temp, 2, rp.wtemp);
	(*rp_eval)(k, rp.ytemp, xtree->response_est, &(xtree->risk),
		     rp.wtemp);
	xtree->complexity = xtree->risk;
	partition(1, xtree, &temp);
	fix_cp(xtree, xtree->complexity);

	/*
	** run the extra data down the new tree
	*/
	for (j=0; j<rp.n; j++) {
	    if (rp.which[j]==0) {
		rundown2(xtree, j, cp, (predict+ j* *ncp));
		}
	    }
	free_tree(xtree, 1);
	}
    }
/* SCCS @(#)surrogate.c	1.6 12/13/99 */
/*
** Calculate the surrogate splits for a node and its primary
**    (This routine is an awful lot like bsplit)
**
** Input :      node
**              start and stop indices for the arrays (which obs apply)
**
** Output:      Fills in the node's
**                      surrogate splits
**                      lastsurrogate value
**
** Uses:        The global vector tempvec (integer) as a temporary, assumed
**                to be of length n.
*/

static
void surrogate(struct node *me, int nodenum)
    {
    int i, j, k;
    int var;   /* the primary split variable */
    FLOAT split;
    double improve;
    double lcount, rcount;    /* weight sent left and right by primary */
    unsigned int extra;
    struct split *ss;
    int  *index;
    int  *which,
	 *tempy;
    long **sorts;
    FLOAT **xdata;
    int ncat;
    double adj_agree;

    which = rp.which;
    tempy = rp.tempvec;
    sorts = rp.sorts;
    xdata = rp.xdata;
    /*
    ** First construct, in tempy, the "y" variable for this calculation.
    ** It will be LEFT:goes left, 0:missing, RIGHT:goes right.
    **  Count up the number of obs the primary sends to the left, as my
    **  last surrogate (or to the right, if larger).
    */
    var = (me->primary)->var_num;
    if (rp.numcat[var]==0)  {  /* continuous variable */
	split = (me->primary)->spoint;
	extra = (me->primary)->csplit[0];
	for (i=0; i<rp.n; i++) {
	    j = sorts[var][i];
	    if (j<0) tempy[-(j+1)]=0;
	    else if (which[j] == nodenum) {
		if (xdata[var][i] < split)
			 tempy[j] = extra;
		else
			 tempy[j] =  -extra;
		}
	    }
	}

    else {  /* categorical variable */
	index = (me->primary)->csplit;
	for (i=0; i<rp.n; i++) {
	    if (which[i] != nodenum) continue;
	    if (sorts[var][i]<0) tempy[i] =0;
	    else        tempy[i] = index[(int)xdata[var][i] -1];
	    }
	}

    lcount=0; rcount=0;
    for (i=0; i<rp.n; i++) {
	if (which[i] != nodenum) continue;
	switch(tempy[i]) {
	    case LEFT : lcount += rp.wt[i];  break;
	    case RIGHT: rcount += rp.wt[i];  break;
	    default: break;
	    }
	}

    if (lcount < rcount) me->lastsurrogate = RIGHT;
    else                 me->lastsurrogate = LEFT;

    /*
    ** Now walk through the variables
    */
    me->surrogate =0;
    for (i=0; i<rp.nvar; i++) {
	if (var == i) continue;
	ncat = rp.numcat[i];

	choose_surg(nodenum, tempy, xdata[i], sorts[i], ncat,
		       &improve, &split, rp.csplit,   lcount, rcount,
		       &adj_agree);
	if (adj_agree <=0) continue;  /*no better than default */

	/*  sort it onto the list of surrogates */
	ss = insert_split( &(me->surrogate), ncat, improve, rp.maxsur);
	if (ss !=0) {
	    ss->improve  = improve;
	    ss->var_num   = i;
	    ss->count     = 0;       /*corrected by nodesplit() */
	    ss->adj       = adj_agree;
	    if (rp.numcat[i]==0) {
		ss->spoint    = split;
		ss->csplit[0] = rp.csplit[0];
		}
	    else for (k=0; k<rp.numcat[i]; k++) ss->csplit[k] = rp.csplit[k];
	    }
	}
    }
/* SCCS  @(#)xval.c	1.8 12/13/99 */
/*
** Cross validate a model.  This routine is responsible for filling in
**  two vectors -- xrisk = cross-validated risk estimate
**                 xstd  = std of xrisk
**
** Basic method is to use a stratified partitioning of the data (NOT random)
**  into n_xval subgroups.  One by one, each of these groups is left out of
**  the partitioning by setting 'which' to 0.  After partitioning, the risk
**  of each left out subject is determined, under each of the unique
**  complexity parameters.
** The x-groups are set by the calling S-routine, so they can actually be
**  random, non-random, or whatever, as far as this routine is concerned.
**
**  n_xval: number of cross-validation subsets
**  cptable: head of the complexity parameter table, were results will be
**              stored
**  x_grp(n): defines the groups.  Integers from 1 to n_xval
**  maxcat  : max # categories, in any given categorical variable
**  error   : possible error message
**  parms   : vector of input parameters, initializers for the splitting rule
*/

static int debug =0;    /*if it is odd, print out every tree */
                        /*if >= 2, print out every risk value we see */
/* Next line only if mainline version */
#ifdef MAIN
extern char *xname[];
#endif

static
void xval(int n_xval,  struct cptable *cptable_head,  long *x_grp, 
	  int maxcat,  char **error,                  double * parms)
    {
    int i,j,k, jj;
    double *xtemp, *xpred;
    int    *savew;
    double *cp;
    double alphasave;
    struct node *xtree;
    struct cptable *cplist;
    double temp;
    double old_n, old_wt, total_wt;
    int *which;

    alphasave = rp.alpha;
    which = rp.which;
    /*
    ** Allocate a set of temporary arrays
    */
    xtemp = (double *) CALLOC(3*rp.num_unique_cp, sizeof(double));
    xpred = xtemp + rp.num_unique_cp;
    cp    = xpred + rp.num_unique_cp;
    savew = (int *)   CALLOC(rp.n, sizeof(int));
    for (i=0; i<rp.n; i++) savew[i] = rp.which[i];

    /*
    ** Make the list of CPs that I will compare against
    */
    cp[0] = 10* cptable_head->cp;    /*close enough to infinity */
    i=1;
    for (cplist= cptable_head; i<rp.num_unique_cp; cplist = cplist->forward) {
	cp[i] = sqrt(cplist->cp * (cplist->forward)->cp);
	i++;
	}
    old_n =rp.n;
    total_wt =0;
    for (i=0; i<rp.n; i++) total_wt += rp.wt[i];
    old_wt = total_wt;

    /*
    ** do the validations
    */
    for (i=0; i<n_xval; i++) {
	/*
	** mark the "leave out" data as fictional node 0, the rest as node 1
	*/
	k=0;
	temp =0;
	for (j=0; j<rp.n; j++) {
	    if (x_grp[j]==(i+1)) {
		which[j] =0;
		}
	    else {
		which[j] =1;
		rp.ytemp[k] = rp.ydata[j];
		rp.wtemp[k] = rp.wt[j];
		k++;
		temp += rp.wt[j];
		}
	    }

	/* rescale the cp */
	for (j=0; j<rp.num_unique_cp; j++) cp[j] *= temp/old_wt;
	rp.alpha *= temp/old_wt;
	old_wt = temp;

	/*
	** partition the new tree
	*/
	xtree = (struct node *) CALLOC(1, nodesize);
	xtree->num_obs = k;
	(*rp_init)(k,rp.ytemp, maxcat, error, parms, &temp, 2, rp.wtemp);
	(*rp_eval)(k, rp.ytemp, xtree->response_est, &(xtree->risk),
		   rp.wtemp);
	xtree->complexity = xtree->risk;
	partition(1, xtree, &temp);
	fix_cp(xtree, xtree->complexity);
#ifdef MAIN
	if (debug%2 ==1) print_tree(xtree, 1, xname,0,0);
#endif
	/*
	** run the extra data down the new tree
	*/
	for (j=0; j<rp.n; j++) {
	    if (which[j]==0) {
		rundown(xtree, j, cp, xpred, xtemp);
		if (debug >1) {
		   jj = j+1;
		   printf("\nObs %d, y=%f \n", jj, rp.ydata[j][0]);
		   }
		/* add it in to the risk */
		cplist = cptable_head;
		for (jj = 0; jj<rp.num_unique_cp; jj++) {
		    cplist->xrisk += xtemp[jj] * rp.wt[j];
		    cplist->xstd  += xtemp[jj]*xtemp[jj] * rp.wt[j];
		    if (debug>1) printf("  cp=%f, pred=%f, xtemp=%f\n",
					  cp[jj]/old_wt, xpred[jj], xtemp[jj]);
		    cplist = cplist->forward;
		    }
		}
	    }
	free_tree(xtree, 1);
	}

    for (cplist = cptable_head; cplist!=0; cplist=cplist->forward) {
	cplist->xstd = sqrt( cplist->xstd -
				      cplist->xrisk* cplist->xrisk/total_wt);
	}
    rp.alpha=alphasave;
    for (i=0; i<rp.n; i++) rp.which[i] = savew[i];
    Free(savew);
    Free(xtemp);
    }
